/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: lmsmtpd_config.c,v 1.19.4.1 2002/11/14 19:40:52 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

/* C includes */
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

/* Library includes */
#include <liblm.h>

/* Project includes */
#include "lmsmtpd_config.h"

/* Local function definitions */
static int lmsmtpd_config_read(int argc, char **argv);
static int lmsmtpd_config_setup_read_number(char *charkey, lm_config_number_t* number, lm_config_number_t defnumber);
static int lmsmtpd_config_setup_read_string(char *charkey, lm_config_string_t* string, lm_config_string_t defstring);
     
int lmsmtpd_config_load(int argc, char **argv)
{
  /* Initialize variables */
  config.smtpgreeting = NULL;
  config.localiphost = NULL;
  config.timeoutsmtpd = LM_SMTPD_DEFAULT_TIMEOUT;
  config.badmailfrom = NULL;
  config.rcpthosts = NULL;
  config.databytes = LM_SMTPD_DEFAULT_DATABYTES;
  config.tcplocalip = NULL;
  config.tcpremoteip= NULL;
  config.tcpremotehost= NULL;
  config.tcpremoteinfo= NULL;
  config.tcprelayclient= NULL;
  config.helohost = NULL;
  config.fakehelohost = 0;
  config.debuglevel = LM_LOG_LEVEL_ERROR;
  config.controldir = NULL;
  config.qmailcontroldir = NULL;
  config.logfile = NULL;
  config.maxlinelen = LM_SMTPD_MAX_LINE_LENGTH_DEFAULT;
  config.maxhops = LM_SMTPD_MAX_HOPS_DEFAULT;
  config.maxrcpt = LM_SMTPD_MAX_RCPT_DEFAULT;
  config.rcptdelayat = LM_SMTPD_RCPT_DELAY_AT_DEFAULT;
  config.rcptdelayinc = LM_SMTPD_RCPT_DELAY_INC_DEFAULT;
  config.rcptdelaymax = LM_SMTPD_RCPT_DELAY_MAX_DEFAULT;
  config.check_vdomain_prog = NULL;
  
  lmsmtpd_config_read(argc, argv);
  
  return 0;
}

/* Clean up allocated memory */
int lmsmtpd_config_cleanup(void)
{

  /* TODO? check for NULL? */
  LM_STRING_FREE(config.me);
  LM_STRING_FREE(config.smtpgreeting);
  LM_STRING_FREE(config.localiphost);
  LM_STRING_FREE(config.tcpremoteip);
  LM_STRING_FREE(config.tcplocalip);
  LM_STRING_FREE(config.rcpthosts);
  LM_STRING_FREE(config.badmailfrom);
  LM_STRING_FREE(config.tcpremotehost);
  LM_STRING_FREE(config.tcpremoteinfo);
  LM_STRING_FREE(config.tcprelayclient);
  LM_STRING_FREE(config.helohost);
  LM_STRING_FREE(config.controldir);
  LM_STRING_FREE(config.qmailcontroldir);
  LM_STRING_FREE(config.logfile);
  LM_STRING_FREE(config.virtualdomains);
  LM_STRING_FREE(config.check_vdomain_prog);
  
  return 0;
}

/* Program globals */
static int lmsmtpd_config_setup_read_string(char *charkey, lm_config_string_t* string, lm_config_string_t defstring)
{
  int retval;
  lm_config_key_t key = NULL;
  lm_config_string_t retstring = NULL;
  
  LM_STRING_NEW(key, charkey);
  retval = LM_CONFIG_READ_STRING(key, retstring, defstring);
  LM_STRING_FREE(key);
  *string = retstring;
  return retval;
}

static int lmsmtpd_config_setup_read_number(char *charkey, lm_config_number_t* number, lm_config_number_t defnumber)
{
  int retval;
  lm_config_key_t key = NULL;
  lm_config_number_t retnumber;
  
  LM_STRING_NEW(key, charkey);
  retval = LM_CONFIG_READ_NUMBER(key, retnumber, defnumber);
  LM_STRING_FREE(key);
  *number = retnumber;
  return retval;
}

static int lmsmtpd_config_read(int argc, char **argv)
{
  lm_config_string_t defstring = NULL;
  lm_config_string_t defunknown = NULL;
  lm_config_string_t tcplocaliptmp = NULL;
  lm_config_string_t logdir = NULL;
  lm_config_string_t morercpthosts = NULL;
  lm_string_t newline = NULL;

  LM_STRING_NEW(newline, "\n");

  /* Set the log file to /dev/null until we know the actual 
   * log directory, log file name and log level */
  LM_LOG_INIT("/dev/null");

  /* Don't print any log messages until after we've read the debug flag */
  LM_LOG_SET_LEVEL(LM_LOG_LEVEL_NONE);

  /* Open the env config */
  if(LM_CONFIG_ENV_SET_PARAM)
  {
	  LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
			  "could not open environment config -- serious problem!");
  }

  /* Read the log directory if set */
  LM_STRING_NEW(defstring, LM_SMTPD_LOG_DIR_DEFAULT);
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_LOG_DIR,
			  &logdir, defstring))
	  return -1;
  LM_STRING_FREE(defstring);

  /* Read the log file if set */
  LM_STRING_NEW(defstring, LM_SMTPD_LOG_FILE_DEFAULT);
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_LOG_FILE,
			  &config.logfile, defstring))
	  return -1;
  LM_STRING_FREE(defstring);

  LM_STRING_CAT(logdir, config.logfile);
  LM_STRING_FREE(config.logfile);
  config.logfile = logdir;

  /* Read the lmsmptd debug level env variable */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_DEBUG_LEVEL,
			  &config.debuglevel, config.debuglevel))
	  return -1;

  /* Re init the logger with teh actual log file and set the debug level */
  LM_LOG_INIT(LM_STRING_BUFFER(config.logfile));
  LM_LOG_SET_LEVEL(config.debuglevel);

  /* Read the control directory if set */
  LM_STRING_NEW(defstring, LM_SMTPD_CONTROL_DIR_DEFAULT);
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_CONTROL_DIR,
			  &config.controldir, defstring))
	  return -1;
  LM_STRING_FREE(defstring);

  /* Read the qmail control directory if set */
  LM_STRING_NEW(defstring, LM_SMTPD_QMAIL_CONTROL_DIR_DEFAULT);
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_QMAIL_CONTROL_DIR,
			  &config.qmailcontroldir, defstring))
	  return -1;
  LM_STRING_FREE(defstring);

  /* Read the max line length */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_MAX_LINE_LENGTH,
			  &config.maxlinelen, LM_SMTPD_MAX_LINE_LENGTH_DEFAULT))
	  return -1;

  /* Read the max number of hops before flagging error */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_MAX_HOPS,
			  &config.maxhops, LM_SMTPD_MAX_HOPS_DEFAULT))
	  return -1;

  /* Read the max number of rcpts variable */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_MAX_RCPT,
			  &config.maxrcpt, LM_SMTPD_MAX_RCPT_DEFAULT))
	  return -1;
  /* RFC Says we have to have at least 100 recipients */
  if(config.maxrcpt < LM_SMTPD_MAX_RCPT_DEFAULT)
  {
	  LM_LOG_WARN(LM_SMTPD_LOG_SYS, "setting max rcpt to default...user setting too low for RFC");
	  config.maxrcpt = LM_SMTPD_MAX_RCPT_DEFAULT;
  }

  /* Read the number of rcpts to start delaying at */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RCPT_DELAY_AT,
			  &config.rcptdelayat, LM_SMTPD_RCPT_DELAY_AT_DEFAULT))
	  return -1;


  /* Read the amount of seconds to delay each rcpt by after DELAY_AT has been met */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RCPT_DELAY_INC,
			  &config.rcptdelayinc, LM_SMTPD_RCPT_DELAY_INC_DEFAULT))
	  return -1;

  /* Read the amount amount of seconds to delay each rcpt by after DELAY_AT has been met */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RCPT_DELAY_MAX,
			  &config.rcptdelaymax, LM_SMTPD_RCPT_DELAY_MAX_DEFAULT))
	  return -1;

  /* Read the max virtual domain user verification program variable */
  LM_STRING_NEW(defstring, "");
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_VDOMAIN_CHECKUSER_PROG,
			  &config.check_vdomain_prog, defstring))
	  return -1;
  LM_STRING_FREE(defstring);

  /* Close the env config */
  LM_CONFIG_CLOSE;

  /* Open the qmail control directory and read the qmail control files */
  if(LM_CONFIG_CONTROL_SET_PARAM(config.qmailcontroldir))
  {
	  LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
			  "could not open config control directory!");
  }

  /* Setup a blank string for defaults values */
  LM_STRING_NEW(defstring, "");
  LM_STRING_NEW(defunknown, "unknown");

  /* Read me from the config */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_ME,
			  &config.me, defstring))
	  return -1;
  /* This has to be present for qmail to function properly */
  if(LM_STRING_LEN(config.me) == 0)
	  return -1;

  /* Read the smtpgreeting from the config */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_SMTPGREETING,
			  &config.smtpgreeting, config.me))
	  return -1;

  /* Read the localiphost from the config */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_LOCALIPHOST,
			  &config.localiphost, defstring))
	  return -1;

  /* Read the timeoutsmtpd from the config */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_CONTROL_KEY_TIMEOUTSMTPD,
			  &config.timeoutsmtpd, LM_SMTPD_DEFAULT_TIMEOUT))
	  return -1;
  /* QMAIL */
  if(config.timeoutsmtpd < 0)
	  config.timeoutsmtpd = 1;

  /* Read rcpthosts control file */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_RCPTHOSTS,
			  &config.rcpthosts, defstring))
	  return -1;

  /* Read MORERCPTHOSTS and cat it onto the end of RCPTHOSTS */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_MORERCPTHOSTS,
			  &morercpthosts, defstring))
	  return -1;

  LM_STRING_CAT(config.rcpthosts, newline);
  LM_STRING_CAT(config.rcpthosts, morercpthosts);
  LM_STRING_FREE(morercpthosts);

  /* Read virtualdomains control file */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_VIRTUALDOMAINS,
			  &config.virtualdomains, defstring))
	  return -1;

  /* Read badmailfrom control file */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_CONTROL_KEY_BADMAILFROM,
			  &config.badmailfrom, defstring))
	  return -1;

  /* Read the databytes from the config */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_CONTROL_KEY_DATABYTES,
			  &config.databytes, LM_SMTPD_DEFAULT_DATABYTES))
	  return -1;

  /* Close the control current config */
  LM_CONFIG_CLOSE;

  /* Open the env config */
  if(LM_CONFIG_ENV_SET_PARAM)
  {
	  LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
			  "could not open environment config -- serious problem!");
  }

  /* Read the databytes from the environment */
  /* defaults to current value if not present in environment */
  if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_KEY_DATABYTES,
			  &config.databytes, config.databytes))
	  return -1;
  /* QMAIL */
  if(config.databytes == UINT_MAX)
	  config.databytes--;

  /* Read the remote ip given by TCP server */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_KEY_TCPREMOTEIP,
			  &config.tcpremoteip, defunknown))
	  return -1;

  /* Set the variable localip to TCPLOCALHOST if set, else set it 
	 to TCPLOCALIP, else set it to unknown */
  /* Read the remote ip given by TCP server */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_KEY_TCPLOCALHOST,
			  &tcplocaliptmp, defunknown))
	  return -1;
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_KEY_TCPLOCALIP,
			  &config.tcplocalip, tcplocaliptmp))
	  return -1;
  LM_STRING_FREE(tcplocaliptmp);

  /* Read tcp remote hostname given by TCP server */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_KEY_TCPREMOTEHOST,
			  &config.tcpremotehost, defunknown))
	  return -1;

  /* Read tcp remote host info given by TCP server */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_KEY_TCPREMOTEINFO,
			  &config.tcpremoteinfo, defstring))
	  return -1;

  /* Read relay client info given by TCP server */
  if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_KEY_RELAYCLIENT,
			  &config.tcprelayclient, defstring))
	  return -1;

  /* Close the env config */
  LM_CONFIG_CLOSE;

  /* Build the server hello string from the argument given by tcp server */
  LM_STRING_DUP(config.helohost, config.tcpremotehost);
  config.fakehelohost = 0;	/* Real helo host given  */

  /* Free up remaining memory */
  LM_STRING_FREE(newline);
  LM_STRING_FREE(defstring);
  LM_STRING_FREE(defunknown);
  return 0;
}


