/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: lmsmtpd_config.h,v 1.15 2002/11/12 19:41:49 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <liblm.h>

#ifndef _LMSMTPD_CONFIG_H
#define _LMSMTPD_CONFIG_H

/* Program constants */
#define LM_SMTPD_CONTROL_DIR_DEFAULT "/var/qmail/control/" /* Remember to put trailing /  */
#define LM_SMTPD_QMAIL_CONTROL_DIR_DEFAULT "/var/qmail/control/" /* Remember to put trailing /  */
#define LM_SMTPD_LOG_DIR_DEFAULT "/var/log/"
#define LM_SMTPD_LOG_FILE_DEFAULT "lmsmptd.log"
#define LM_SMTPD_LOG_OUT stderr
#define LM_SMTPD_LOG_SYS "lmsmptd"

#define LM_SMTPD_MAX_LINE_LENGTH_DEFAULT 1024
#define LM_SMTPD_MAX_SMTP_CMD_LENGTH_DEFAULT LM_SMTPD_MAX_LINE_LENGTH

#define LM_SMTPD_MAX_RCPT_DEFAULT 100
#define LM_SMTPD_RCPT_DELAY_AT_DEFAULT 10
#define LM_SMTPD_RCPT_DELAY_INC_DEFAULT 1
#define LM_SMTPD_RCPT_DELAY_MAX_DEFAULT 10

#define LM_SMTPD_MAX_HOPS_DEFAULT 100

/* Default constants for application */
#define LM_SMTPD_DEFAULT_TIMEOUT 1200
#define LM_SMTPD_DEFAULT_DATABYTES 64*1024

/* Key strings for config and environment variables */
#define LM_SMTPD_CONTROL_KEY_ME "me"
#define LM_SMTPD_CONTROL_KEY_SMTPGREETING "smtpgreeting"
#define LM_SMTPD_CONTROL_KEY_LOCALIPHOST "localiphost"
#define LM_SMTPD_CONTROL_KEY_TIMEOUTSMTPD "timeoutsmtpd"
#define LM_SMTPD_CONTROL_KEY_RCPTHOSTS "rcpthosts"
#define LM_SMTPD_CONTROL_KEY_MORERCPTHOSTS "morercpthosts"
#define LM_SMTPD_CONTROL_KEY_BADMAILFROM "badmailfrom"
#define LM_SMTPD_CONTROL_KEY_DATABYTES "databytes"
#define LM_SMTPD_CONTROL_KEY_VIRTUALDOMAINS "virtualdomains"
#define LM_SMTPD_CONTROL_KEY_LOCALS "locals"

#define LM_SMTPD_ENV_KEY_DATABYTES "DATABYTES"
#define LM_SMTPD_ENV_KEY_TCPREMOTEIP "TCPREMOTEIP"
#define LM_SMTPD_ENV_KEY_TCPLOCALHOST "TCPLOCALHOST"
#define LM_SMTPD_ENV_KEY_TCPLOCALIP "TCPLOCALIP"
#define LM_SMTPD_ENV_KEY_TCPREMOTEHOST "TCPREMOTEHOST"
#define LM_SMTPD_ENV_KEY_TCPREMOTEINFO "TCPREMOTEINFO"
#define LM_SMTPD_ENV_KEY_RELAYCLIENT "RELAYCLIENT"

/* Debug message and log level: see lmlogger.h for exact values LM_LOG_LEVEL_*
   0 no message
   1 errors
   2 warnings
   3 all
*/   
#define LM_SMTPD_ENV_DEBUG_LEVEL "LMSMTPD_DEBUG"
#define LM_SMTPD_ENV_CONTROL_DIR "LMSMTPD_CONTROLDIR"
#define LM_SMTPD_ENV_QMAIL_CONTROL_DIR "LMSMTPD_QMAILCONTROLDIR"
#define LM_SMTPD_ENV_LOG_DIR "LMSMTPD_LOGDIR"
#define LM_SMTPD_ENV_LOG_FILE "LMSMTPD_LOGFILE"
#define LM_SMTPD_ENV_MAX_LINE_LENGTH "LMSMTPD_MAX_LINE_LENGTH"
#define LM_SMTPD_ENV_MAX_RCPT "LMSMTPD_MAX_RCPT"
#define LM_SMTPD_ENV_RCPT_DELAY_AT "LMSMTPD_RCPT_DELAY_AT"
#define LM_SMTPD_ENV_RCPT_DELAY_INC "LMSMTPD_RCPT_DELAY_INC"
#define LM_SMTPD_ENV_RCPT_DELAY_MAX "LMSMTPD_RCPT_DELAY_MAX"
#define LM_SMTPD_ENV_MAX_HOPS "LMSMTPD_MAX_HOPS"
#define LM_SMTPD_ENV_VDOMAIN_CHECKUSER_PROG "LMSMTPD_VDOMAIN_CHECKUSER_PROG"

typedef struct
{
    /* Variables from control files or environment */
  lm_config_string_t me;
  lm_config_string_t smtpgreeting;
  lm_config_string_t localiphost;
  lm_config_number_t timeoutsmtpd;
  lm_config_string_t badmailfrom;
  lm_config_string_t rcpthosts;
  lm_config_string_t virtualdomains;
  lm_config_number_t databytes;
  
  /* Variables passed from tcpserver */
  lm_config_string_t tcplocalip;
  lm_config_string_t tcpremoteip;
  lm_config_string_t tcpremotehost;
  lm_config_string_t tcpremoteinfo;
  lm_config_string_t tcprelayclient;

  /* Constructed variables */
  lm_config_string_t helohost;
  lm_config_bool_t fakehelohost;

  lm_config_number_t debuglevel;
  lm_config_string_t controldir;
  lm_config_string_t qmailcontroldir;
  lm_config_string_t logfile;
  lm_config_number_t maxlinelen;
  lm_config_number_t maxhops;
  lm_config_number_t maxrcpt;
  lm_config_number_t rcptdelayat;
  lm_config_number_t rcptdelayinc;
  lm_config_number_t rcptdelaymax;
  lm_config_string_t check_vdomain_prog;
} lmsmtpd_config_t;

extern lmsmtpd_config_t config;

int lmsmtpd_config_load(int argc, char **argv);
int lmsmtpd_config_cleanup(void);

#endif /*  _LMSMTPD_CONFIG */

